#include "server/auth.h"
#include "server/response.h"
#include "server/util.h"

#include "db/auth.h"

#include <civetweb.h>

#include <cjson/cJSON.h>

#include <assert.h>
#include <stdlib.h>
#include <string.h>

typedef struct
{
    char* user_id;
    char* password;
} register_form_t;

static void register_form_dtor(register_form_t* form)
{
    if (form->user_id) free(form->user_id);
    if (form->password) free(form->password);
}

static int field_found(const char* key, const char* filename, char* path, size_t pathlen, void* user_data)
{
    return MG_FORM_FIELD_HANDLE_GET;
}

static int field_get(const char* key, const char* value, size_t valuelen, void* user_data)
{
    register_form_t* form = (register_form_t*)user_data;
    if (strcmp(key, "user_id") == 0) {
        form->user_id = kqm_strndup(value, valuelen);
    } else if (strcmp(key, "password") == 0) {
        form->password = kqm_strndup(value, valuelen);
    }
    if (form->user_id && form->password) {
        return MG_FORM_FIELD_HANDLE_ABORT;
    }
    return MG_FORM_FIELD_HANDLE_GET;
}


int user_register_handler(mg_connection* conn, void* cbdata)
{
    const mg_request_info* post_body = mg_get_request_info(conn);

    if (post_body == NULL) {
        res_null_req(conn);
        return 1;
    }

    if (strcmp(post_body->request_method, "POST")) {
        res_must_post(conn);
        return 1;
    }

    register_form_t form = {NULL, NULL};

    mg_form_data_handler register_callback = {
        .field_found = field_found,
        .field_get   = field_get,
        .field_store = NULL,
        .user_data   = &form,
    };

    mg_handle_form_request(conn, &register_callback);

    if (!form.user_id) {
        res_need_user_id(conn);
        register_form_dtor(&form);
        return 1;
    }
    if (!form.password) {
        res_need_password(conn);
        register_form_dtor(&form);
        return 1;
    }

    int result;
    int flag = check_user_exists(form.user_id, &result);
    if (!flag) {
        res_check_exist_fail(conn);
        register_form_dtor(&form);
        return 1;
    } else if (result) {
        res_user_exist(conn);
        register_form_dtor(&form);
        return 1;
    }

    flag = registe(form.user_id, form.password);
    if (!flag) {
        res_register_fail(conn);
    } else {
        res_register(conn);
    }
    register_form_dtor(&form);
    return 1;
}
